@@ -146,6 +146,26 @@ class AndroidCommand(Command):
         test_dir = os.path.join(self.config.installdir_android, test_id)
         return self.adb.exists(test_dir)
 
+    def assertDeviceIsConnected(self):
+        if not self.adb.isDeviceConnected():
+            if self.adb.serial:
+                raise Exception("Device '%s' is not connected" %
+                                       self.adb.serial)
+            else:
+                raise Exception("No device found")
+
+    def invoke(self):
+        self.adb = ADB(self.args.serial)
+        try:
+            self.assertDeviceIsConnected()
+        except Exception as err:
+            raise LavaCommandError(err)
+
+        self.invoke_sub()
+
+    def invoke_sub():
+        raise NotImplementedError
+
 
 class AndroidTestCommand(AndroidCommand):
     @classmethod
@@ -186,8 +206,7 @@ class list_installed(AndroidCommand):
     program:: lava-android-test list-tests
     program:: lava-android-test list-tests -s device_serial
     """
-    def invoke(self):
-        self.adb = ADB(self.args.serial)
+    def invoke_sub(self):
 
         self.say("Installed tests:")
         try:
@@ -207,8 +226,7 @@ class list_results(AndroidCommand):
     program:: lava-android-test list-results
     program:: lava-android-test list-results -s device_serial
     """
-    def invoke(self):
-        self.adb = ADB(self.args.serial)
+    def invoke_sub(self):
         self.say("Saved results:")
         try:
             (ret_code, output) = self.adb.listdir(
@@ -233,14 +251,13 @@ class install(AndroidTestCommand):
         super(cls, install).register_arguments(parser)
         parser.add_argument('-o', '--install-option')
 
-    def invoke(self):
+    def invoke_sub(self):
         tip_msg = self.get_tip_msg("Install test")
         self.say_begin(tip_msg)
 
-        self.adb = ADB(self.args.serial)
         if self.test_installed(self.args.test_id):
-            raise LavaCommandError(
-                    "The test (%s) has already installed." % self.args.test_id)
+            raise LavaCommandError("The test (%s) has already installed." %
+                                   self.args.test_id)
         test = testloader(self.args.test_id, self.args.serial)
         try:
             test.install(self.args.install_option)
@@ -256,7 +273,7 @@ class uninstall(AndroidTestCommand):
     program:: lava-android-test uninstall test-id
     program:: lava-android-test uninstall test-id -s device_serial
     """
-    def invoke(self):
+    def invoke_sub(self):
         tip_msg = self.get_tip_msg("Uninstall test")
         self.say_begin(tip_msg)
 
@@ -292,11 +309,14 @@ class run(AndroidTestCommand):
                                  " bundle and finally save the complete bundle"
                                  " to the  specified FILE."))
 
-    def invoke(self):
+    def invoke_sub(self):
         tip_msg = self.get_tip_msg("Run test")
         self.say_begin(tip_msg)
 
-        self.adb = ADB(self.args.serial)
+        if not self.test_installed(self.args.test_id):
+            raise LavaCommandError(
+                "The test (%s) has not been installed yet." %
+                self.args.test_id)
         test = testloader(self.args.test_id, self.args.serial)
 
         if not self.test_installed(test.testname):
@@ -324,8 +344,7 @@ class run(AndroidTestCommand):
 class run_custom(AndroidCommand):
     """
     Run the command(s) that specified by the -c option in the command line
-    program:: lava-android-test run-custom -c 'command1' -c 'command2'
-     -p 'parse-regex1'
+    program:: lava-android-test run-custom -c 'command1' -c 'command2' -p 'parse-regex1'
     program:: lava-android-test run test-id -s device_serial
     program:: lava-android-test run test-id -s device_serial -o outputfile
     """
@@ -353,10 +372,9 @@ class run_custom(AndroidCommand):
                                  " bundle and finally save the complete bundle"
                                  " to the  specified FILE."))
 
-    def invoke(self):
+    def invoke_sub(self):
 
         config = get_config()
-        self.adb = ADB(self.args.serial)
         test_name = 'custom'
         ADB_SHELL_STEPS = []
         STEPS_HOST_PRE = []
@@ -440,7 +458,7 @@ class parse(AndroidResultsCommand):
     Parse the results of previous test that run on the specified device
     program:: lava-android-test parse test-result-id
     """
-    def invoke(self):
+    def invoke_sub(self):
         bundle = generate_combined_bundle(self.args.serial,
                                           self.args.result_id)
         try:
@@ -462,7 +480,7 @@ class parse_custom(AndroidResultsCommand):
                             help=("Specified the regular expression used"
                                   " for analyzing command output"))
 
-    def invoke(self):
+    def invoke_sub(self):
         PATTERN = None
         if self.args.parse_regex:
             PATTERN = self.args.parse_regex
@@ -473,7 +491,7 @@ class parse_custom(AndroidResultsCommand):
         test = AndroidTest(testname=test_name, installer=inst,
                                 runner=run, parser=parser)
         test.parser.results = {'test_results': []}
-        test.setadb(ADB(self.args.serial))
+        test.setadb(self.adb)
 
         bundle = generate_combined_bundle(self.args.serial,
                                           self.args.result_id, test=test)
@@ -563,8 +581,7 @@ class show(AndroidResultCommand):
     program:: lava-android-test show result-id
     program:: lava-android-test show result-id -s device_serial
     """
-    def invoke(self):
-        self.adb = ADB(self.args.serial)
+    def invoke_sub(self):
         resultsdir = os.path.join(self.config.resultsdir_android,
                                    self.args.result_id)
         if not self.adb.exists(resultsdir):
@@ -608,18 +625,18 @@ class rename(AndroidResultCommand):
         parser.add_argument("result_id_new",
                             help="New test result identifier")
 
-    def invoke(self):
+    def invoke_sub(self):
         srcdir = os.path.join(self.config.resultsdir_android,
                                self.args.result_id)
         destdir = os.path.join(self.config.resultsdir_android,
                                 self.args.result_id_new)
-        adb = ADB(self.args.serial)
-        if not adb.exists(srcdir):
+
+        if not self.adb.exists(srcdir):
             self.say("Result (%s) not found" % self.args.result_id)
             return
-        if adb.exists(destdir):
+        if self.adb.exists(destdir):
             self.say("Destination result name already exists")
-        adb.move(srcdir, destdir)
+        self.adb.move(srcdir, destdir)
 
 
 class remove(AndroidResultsCommand):
@@ -638,9 +655,9 @@ class remove(AndroidResultsCommand):
                             action="store_true",
                             help=("give an interactive question about remove"))
 
-    def remove(self, adb, rid):
+    def remove(self, rid):
         resultsdir = os.path.join(self.config.resultsdir_android, rid)
-        if not adb.exists(resultsdir):
+        if not self.adb.exists(resultsdir):
             self.say("No result found for '%s'" % rid)
             return
         if not self.args.force:
@@ -648,9 +665,8 @@ class remove(AndroidResultsCommand):
             response = raw_input()
             if response[0].upper() != 'Y':
                 return
-        adb.rmtree(resultsdir)
+        self.adb.rmtree(resultsdir)
 
-    def invoke(self):
-        adb = ADB(self.args.serial)
+    def invoke_sub(self):
         for rid in self.args.result_id:
-            self.remove(adb, rid)
+            self.remove(rid)
